
# q-expansion of newform 1274.2.d.c, downloaded from the LMFDB on 09 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1274
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[885, [1, 0]], [197, [-1, 0]]]
aps_data = [[0, 1], [1, 0], [0, 3], [0, 0], [0, 0], [-2, -3], [-3, 0], [0, 6], [-6, 0], [0, 0], [0, 0], [0, 3], [0, 0], [-1, 0], [0, -3], [-6, 0], [0, 6], [8, 0], [0, -12], [0, 15], [0, -6], [10, 0], [0, -6], [0, 6], [0, 12], [-12, 0], [-14, 0], [-12, 0], [0, 9], [-6, 0], [-2, 0], [3, 0], [0, 18], [-5, 0], [0, -6], [0, -15], [22, 0], [0, 6], [0, 12], [6, 0], [15, 0], [-2, 0], [12, 0], [0, 6], [0, 3], [20, 0], [-23, 0], [0, 9], [0, 12], [0, -9], [-21, 0], [0, 9], [0, 30], [-12, 0], [-3, 0], [24, 0], [0, 0], [0, -15], [8, 0], [0, -30], [-4, 0], [0, 9], [0, -18], [18, 0], [-19, 0], [0, 18], [0, 30], [13, 0], [33, 0], [0, 21], [0, -6], [0, 24], [-8, 0], [4, 0], [0, -6], [0, 9], [-30, 0], [0, -18], [0, 30], [0, 6], [-15, 0], [0, -15], [0, -15], [11, 0], [10, 0], [-21, 0], [0, 24], [0, 18], [0, 15], [0, -24], [12, 0], [0, -39], [0, -12], [27, 0], [0, -36], [6, 0], [0, 6], [-27, 0], [16, 0], [0, -15], [-37, 0], [0, -27], [-39, 0], [45, 0], [-23, 0], [0, 42], [0, -18], [0, -36], [-30, 0], [-37, 0], [22, 0], [0, 6], [0, -12], [0, -24], [0, -15], [-18, 0], [0, -36], [42, 0], [-36, 0], [0, 0], [0, -30], [-1, 0], [-18, 0], [0, 6], [0, 0], [42, 0], [0, -6], [0, 0], [-28, 0], [0, 9], [0, -36], [0, -39], [32, 0], [-2, 0], [0, 30], [0, -24], [0, -21], [0, 12], [-18, 0], [15, 0], [0, 0], [0, 45], [14, 0], [0, 18], [20, 0], [0, -24], [0, 39], [-18, 0], [-40, 0], [0, -39], [0, 3], [33, 0], [-11, 0], [12, 0], [-17, 0], [12, 0], [-20, 0], [0, 36], [2, 0], [0, 45], [0, 48], [9, 0], [0, 3], [-27, 0], [0, -12], [0, 9], [2, 0], [-8, 0], [0, -6], [36, 0], [0, 24], [0, -45], [12, 0], [0, -24], [40, 0], [45, 0], [0, 0], [0, 30], [16, 0], [-10, 0], [0, -12], [-12, 0], [4, 0], [0, 42], [0, -24], [-30, 0], [38, 0], [0, -6], [0, 24], [0, 0], [-29, 0], [0, -24], [47, 0], [0, 30], [-27, 0], [-54, 0], [0, 60], [-26, 0], [0, 42], [-54, 0], [0, -21], [-62, 0], [0, 57], [55, 0], [0, -24], [42, 0], [0, -39], [-36, 0], [0, 24], [23, 0], [-2, 0], [-12, 0], [-26, 0], [0, 42], [0, -39], [0, -30], [-32, 0], [63, 0], [0, -72], [0, 21], [32, 0], [0, -9], [0, -36], [0, -9], [27, 0], [-10, 0], [-39, 0], [30, 0], [62, 0], [0, 60], [14, 0], [-20, 0], [0, 0], [-3, 0], [1, 0], [0, 27], [0, 66], [0, -39], [-60, 0], [-12, 0], [0, 66], [-7, 0], [-4, 0], [0, 39], [0, -36], [30, 0], [0, -33], [0, 60], [0, -36], [-6, 0], [0, 63], [0, 0], [0, -27], [35, 0], [66, 0], [0, 24], [-38, 0], [0, -18], [42, 0], [0, -18], [64, 0], [33, 0], [0, 21], [16, 0], [0, -48], [35, 0], [0, 9], [0, -30], [0, 36], [-36, 0], [-22, 0], [0, -42], [0, -36], [50, 0], [-58, 0], [0, 24], [0, 78], [0, 54], [0, 0], [63, 0], [6, 0], [0, 45], [-48, 0], [0, 30], [0, 42], [-78, 0], [-11, 0], [0, 3], [0, 21], [-30, 0], [-48, 0], [-33, 0], [0, 36], [0, 0], [44, 0], [-60, 0], [28, 0], [36, 0], [-15, 0], [0, 12], [31, 0], [0, 3], [20, 0]]
